@@ -1,3 +1,20 @@
+#' Add transparency level to hex colors
+#'
+#' @importFrom grDevices col2rgb rgb
+#'
+#' @return Color hex value (with transparency)
+#'
+#' @keywords internal
+rgb2alpha = function(raw_col, alpha) {
+
+  raw_col_rgb = col2rgb(raw_col)
+  alpha_col = rgb(raw_col_rgb[1L, ], raw_col_rgb[2L, ], raw_col_rgb[3L, ],
+                  alpha = alpha * 255L, maxColorValue = 255L)
+
+  alpha_col
+
+}
+
 #' Plot cv.enpls object
 #'
 #' Plot cv.enpls object
@@ -5,6 +22,7 @@
 #' @param x An object of class \code{cv.enpls}.
 #' @param xlim x Vector of length 2 - x axis limits of the plot.
 #' @param ylim y Vector of length 2 - y axis limits of the plot.
+#' @param alpha An alpha transparency value for points, a real number in (0, 1].
 #' @param main Plot title, not used currently.
 #' @param ... Additional graphical parameters, not used currently.
 #'
@@ -29,7 +47,8 @@
 #' cvfit = cv.enpls(x, y, reptimes = 10)
 #' plot(cvfit)
 
-plot.cv.enpls = function(x, xlim = NULL, ylim = NULL, main = NULL, ...) {
+plot.cv.enpls = function(x, xlim = NULL, ylim = NULL, alpha = 0.8,
+                         main = NULL, ...) {
 
   if (!inherits(x, 'cv.enpls'))
     stop('This function only works for objects of class "cv.enpls"')
@@ -48,7 +67,7 @@ plot.cv.enpls = function(x, xlim = NULL, ylim = NULL, main = NULL, ...) {
                         xmin = xlim[1L], xmax = xlim[2L],
                         ymin = ylim[1L], ymax = ylim[2L])) +
     geom_abline(slope = 1, intercept = 0, colour = 'darkgrey') +
-    geom_point(size = 3, shape = 1, alpha = 0.8) +
+    geom_point(size = 3, shape = 1, alpha = alpha) +
     coord_fixed(ratio = 1) +
     xlab('Observed Response') +
     ylab('Predicted Response')
@@ -148,6 +167,7 @@ plot.enpls.fs = function(x, nvar = NULL,
 #' can be \code{"quantile"} or \code{"sd"}.
 #' @param prob Quantile probability as the cut-off value.
 #' @param sdtimes Times of standard deviation as the cut-off value.
+#' @param alpha An alpha transparency value for points, a real number in (0, 1].
 #' @param main Plot title.
 #' @param ... Additional graphical parameters for \code{\link{plot}}.
 #'
@@ -175,7 +195,7 @@ plot.enpls.fs = function(x, nvar = NULL,
 plot.enpls.od = function(x,
                          criterion = c('quantile', 'sd'),
                          prob = 0.05, sdtimes = 3L,
-                         main = NULL, ...) {
+                         alpha = 1, main = NULL, ...) {
 
   if (!inherits(x, 'enpls.od'))
     stop('This function only works for objects of class "enpls.od"')
@@ -212,7 +232,7 @@ plot.enpls.od = function(x,
   # grid lines
   grid(col = 'white', lty = 1)
 
-  points(error.mean, error.sd)
+  points(error.mean, error.sd, col = rgb2alpha('#000000', alpha))
 
   axis(side = 1, labels = TRUE, col = 'darkgrey', lwd = 0.3, cex.axis = 0.8)
   axis(side = 2, labels = TRUE, col = 'darkgrey', lwd = 0.3, cex.axis = 0.8)
@@ -221,24 +241,131 @@ plot.enpls.od = function(x,
   abline(v = vpos, col = 'black', lty = 2)
 
   if (length(yout) != 0L) {
-    points(error.mean[yout], error.sd[yout], pch = 21, bg = '#d62728')
+    points(error.mean[yout], error.sd[yout], pch = 21,
+           col = rgb2alpha('#000000', alpha),
+           bg = rgb2alpha('#D62728', alpha))
     text(error.mean[yout], error.sd[yout],
          labels = as.character(yout),
-         col = '#d62728', cex = 0.8, font = 2, pos = 3)
+         col = '#D62728', cex = 0.8, font = 2, pos = 3)
   }
 
   if (length(Xout) != 0L) {
-    points(error.mean[Xout], error.sd[Xout], pch = 21, bg = '#1f77b4')
+    points(error.mean[Xout], error.sd[Xout], pch = 21,
+           col = rgb2alpha('#000000', alpha),
+           bg = rgb2alpha('#1F77B4', alpha))
     text(error.mean[Xout], error.sd[Xout],
          labels = as.character(Xout),
-         col = '#1f77b4', cex = 0.8, font = 2, pos = 1)
+         col = '#1F77B4', cex = 0.8, font = 2, pos = 1)
   }
 
   if (length(abnormal) != 0L) {
-    points(error.mean[abnormal], error.sd[abnormal], pch = 21, bg = '#2ca02c')
+    points(error.mean[abnormal], error.sd[abnormal], pch = 21,
+           col = rgb2alpha('#000000', alpha),
+           bg = rgb2alpha('#2CA02C', alpha))
     text(error.mean[abnormal], error.sd[abnormal],
          labels = as.character(abnormal),
-         col = '#2ca02c', cex = 0.8, font = 2, pos = 1)
+         col = '#2CA02C', cex = 0.8, font = 2, pos = 1)
   }
 
 }
+
+#' Plot enpls.ad object
+#'
+#' Plot enpls.ad object
+#'
+#' @param x An object of class \code{enpls.ad}.
+#' @param type Plot type. Can be \code{"static"} or \code{"interactive"}.
+#' @param main Plot title, not used currently.
+#' @param ... Additional graphical parameters, not used currently.
+#'
+#' @author Nan Xiao <\url{http://nanx.me}>
+#'
+#' @seealso See \code{\link{enpls.ad}} for model applicability domain
+#' evaluation with ensemble partial least squares regressions.
+#'
+#' @importFrom ggplot2 ggplot scale_shape scale_colour_brewer
+#' @importFrom plotly ggplotly
+#'
+#' @method plot enpls.ad
+#'
+#' @export
+#'
+#' @examples
+#' data("alkanes")
+#' x = alkanes$x
+#' y = alkanes$y
+#'
+#' # training set
+#' x.tr = x[1:100, ]
+#' y.tr = y[1:100]
+#'
+#' # two test sets
+#' x.te = list("test.1" = x[101:150, ],
+#'             "test.2" = x[151:207, ])
+#' y.te = list("test.1" = y[101:150],
+#'             "test.2" = y[151:207])
+#'
+#' set.seed(42)
+#' ad = enpls.ad(x.tr, y.tr, x.te, y.te,
+#'               space = "variable", method = "mc",
+#'               ratio = 0.9, reptimes = 50)
+#' plot(ad)
+#' # The interactive plot requires a HTML viewer
+#' \dontrun{
+#' plot(ad, type = "interactive")}
+
+plot.enpls.ad = function(x,
+                         type = c('static', 'interactive'),
+                         main = NULL, ...) {
+
+  if (!inherits(x, 'enpls.ad'))
+    stop('This function only works for objects of class "enpls.ad"')
+
+  type = match.arg(type)
+
+  n.testset = length(x$'te.error.mean')
+  nsamp.tr  = length(x$'tr.error.mean')
+  nsamp.te  = sapply(x$'te.error.mean', length)
+
+  tr.df = data.frame('Mean' = x$'tr.error.mean',
+                     'SD'   = x$'tr.error.sd',
+                     'Set'  = 'Train')
+
+  te.list = vector('list', n.testset)
+  for (i in 1L:n.testset) {
+    te.list[[i]] = data.frame('Mean' = x[['te.error.mean']][[i]],
+                              'SD'   = x[['te.error.sd']][[i]],
+                              'Set'  = paste0('Test.', i))
+  }
+
+  df = rbind(tr.df, Reduce(rbind, te.list))
+
+  if (type == 'static') {  # static plot with ggplot2
+
+    p = ggplot(df) +
+      geom_point(aes_string(x = 'Mean', y = 'SD',
+                            color = 'Set', shape = 'Set')) +
+      scale_shape(solid = FALSE) +  # hollow shapes
+      scale_colour_brewer(palette = 'Set1') +
+      xlab('Absolute Mean Prediction Error') +
+      ylab('Prediction Error SD')
+
+  } else {  # interactive plot with plotly
+
+    hovertext = sprintf('Sample ID: %s', rownames(df))
+    df = data.frame(df, 'hovertext' = hovertext)
+
+    g = ggplot(df) +
+      geom_point(aes_string(x = 'Mean', y = 'SD',
+                            color = 'Set', text = 'hovertext')) +
+      scale_colour_brewer(palette = 'Set1') +
+      xlab('Absolute Mean Prediction Error') +
+      ylab('Prediction Error SD')
+
+    p = ggplotly(g)
+
+  }
+
+  p
+
+}
